use cast::usize;
use core::fmt::Display;

pub struct InterruptProto {
    irq: u8,
}
impl InterruptProto {
    pub fn dissect(buffer: &[u8]) -> Self {
        Self { irq: buffer[0] }
    }
}

static IRQ_NAMES: [&str; 16] = [
    "Timer", "Keyboard", "Cascade", "COM2", "COM1", "LPT2", "Floppy", "LPT1", "RTC", "Free1",
    "Free2", "Free3", "Mouse", "FPU", "ATA1", "ATA2",
];

impl Display for InterruptProto {
    fn fmt(&self, f: &mut core::fmt::Formatter<'_>) -> core::fmt::Result {
        let irq_name = *IRQ_NAMES.get(usize(self.irq)).unwrap_or(&"Invalid");
        f.write_fmt(format_args!("IRQ {} ({})", self.irq, irq_name))
    }
}
