#![allow(clippy::arithmetic_side_effects, reason = "temp for debugging")]
#![allow(clippy::indexing_slicing, reason = "temp for debugging")]
#![allow(clippy::unwrap_used, reason = "temp for debugging")]

mod errno;
mod file_rpc;
mod hexdump;
mod interrupt_proto;
mod message;
mod proc_man_rpc;
mod rpc_header;
mod syslog_ipc;
mod syslog_msg;
mod syslog_rpc;

use file_rpc::FileRpcMessage;
use hexdump::HexdumpLayer;
use interrupt_proto::InterruptProto;
use message::{DissectedMessage, IpcFrom};
use proc_man_rpc::ProcManRpcMessage;
use rpc_header::RpcHeader;
use syslog_ipc::SyslogIpc;
use syslog_rpc::SyslogRpcMessage;

use crate::serial::SECOND_PORT;

use core::fmt::Write;

pub fn dump_ipc_message(from: usize, dest: usize, buffer: &[u8]) {
    let ipc_proto = u16::from_ne_bytes([buffer[0], buffer[1]]);
    let buffer = &buffer[2..];
    let from = if from == usize::MAX { IpcFrom::Kernel } else { IpcFrom::Process(from) };
    let msg = DissectedMessage::new(from, dest, ipc_proto);
    if ipc_proto == 0 {
        let (rpc_header, buffer) = RpcHeader::dissect(buffer, &msg);
        let msg = msg.set_l1(rpc_header);
        if msg.get_l1().proto == 1 {
            let (file_rpc, rem_data) = FileRpcMessage::dissect(buffer, &msg);
            let msg = msg.set_l2(file_rpc);
            if rem_data.is_empty() {
                writeln!(&*SECOND_PORT, "{msg}").unwrap();
            } else {
                let msg = msg.set_l3(HexdumpLayer::new(&rem_data, 128));
                writeln!(&*SECOND_PORT, "{msg}").unwrap();
            }
        } else if msg.get_l1().proto == 5 {
            let syslog_rpc = SyslogRpcMessage::dissect(buffer, &msg);
            let msg = msg.set_l2(syslog_rpc);
            writeln!(&*SECOND_PORT, "{msg}").unwrap();
        } else if msg.get_l1().proto == 8 {
            let syslog_rpc = ProcManRpcMessage::dissect(buffer, &msg);
            let msg = msg.set_l2(syslog_rpc);
            writeln!(&*SECOND_PORT, "{msg}").unwrap();
        } else {
            let msg = msg.set_l2(HexdumpLayer::new(buffer, 128));
            writeln!(&*SECOND_PORT, "{msg}").unwrap();
        }
    } else if ipc_proto == 1 {
        let msg = msg.set_l1(SyslogIpc::dissect(buffer));
        writeln!(&*SECOND_PORT, "{msg}").unwrap();
    } else if ipc_proto == 2 {
        let msg = msg.set_l1(InterruptProto::dissect(buffer));
        writeln!(&*SECOND_PORT, "{msg}").unwrap();
    } else {
        let msg = msg.set_l1(HexdumpLayer::new(buffer, 128));
        writeln!(&*SECOND_PORT, "{msg}").unwrap();
    }
}
